"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BenchmarkRunner = void 0;
const IMessage_1 = require("../simulator/interface/IMessage");
class BenchmarkRunner {
    constructor(publisher, matchRunner, hillResultMapper) {
        this.publisher = publisher;
        this.matchRunner = matchRunner;
        this.hillResultMapper = hillResultMapper;
    }
    publishEnd(result) {
        this.publisher.queue({
            type: IMessage_1.MessageType.HillEnd,
            payload: result
        });
        this.publisher.publish();
    }
    run(warrior, rules, warriors) {
        const matchResults = [];
        let i;
        for (i = 0; i < warriors.length; i++) {
            if (typeof warriors[i].internalId === 'undefined') {
                warriors[i].internalId = i;
            }
        }
        if (typeof warrior.internalId === 'undefined') {
            warrior.internalId = i;
        }
        for (const warriorB of warriors) {
            matchResults.push(this.matchRunner.run(rules, [warrior, warriorB]));
        }
        const hillResult = this.hillResultMapper.map([warrior, ...warriors], matchResults);
        const result = {
            warriors: hillResult.warriors.filter(x => x.warrior.internalId === warrior.internalId)
        };
        this.publishEnd(result);
        return result;
    }
}
exports.BenchmarkRunner = BenchmarkRunner;
//# sourceMappingURL=BenchmarkRunner.js.map