"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MatchRunner = void 0;
const IMessage_1 = require("../simulator/interface/IMessage");
class MatchRunner {
    constructor(simulator, matchResultMapper, publisher) {
        this.simulator = simulator;
        this.matchResultMapper = matchResultMapper;
        this.publisher = publisher;
    }
    publishEnd(result) {
        this.publisher.queue({
            type: IMessage_1.MessageType.MatchEnd,
            payload: result
        });
        this.publisher.publish();
    }
    run(rules, warriors) {
        for (let i = 0; i < warriors.length; i++) {
            if (typeof warriors[i].internalId === 'undefined') {
                warriors[i].internalId = i;
            }
        }
        const roundResults = [];
        for (let i = 0; i < rules.rounds; i++) {
            this.simulator.initialise(rules.options, warriors);
            const roundResult = this.simulator.run();
            if (!roundResult) {
                throw new Error('Round ended without returning a result');
            }
            roundResults.push(roundResult);
        }
        const result = this.matchResultMapper.map(warriors, roundResults);
        this.publishEnd(result);
        return result;
    }
}
exports.MatchRunner = MatchRunner;
//# sourceMappingURL=MatchRunner.js.map