"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Filter = void 0;
const IToken_1 = require("./interface/IToken");
const PassBase_1 = require("./PassBase");
class Filter extends PassBase_1.PassBase {
    /// <summary>
    /// Filters superfluous tokens from the token stream.
    /// Removes any empty lines and anything after the END preprocessor command
    /// Removes comments
    /// </summary>
    processLine() {
        // Remove empty lines from stream
        // Remove anything after END from stream
        // Remove comments
        const next = this.stream.peek();
        switch (next.category) {
            case IToken_1.TokenCategory.EOL:
            case IToken_1.TokenCategory.Comment:
                this.processEmptyLine();
                break;
            case IToken_1.TokenCategory.Preprocessor:
                if (next.lexeme === 'END') {
                    this.processEnd();
                }
                else {
                    this.processCommentLine();
                }
                break;
            default:
                this.processCommentLine();
                break;
        }
    }
    processEmptyLine() {
        this.stream.readToEOL();
    }
    processEnd() {
        this.processCommentLine();
        this.stream.position = this.stream.tokens.length;
    }
    processCommentLine() {
        let line = this.stream.readToEOL();
        line = line.filter(t => t.category !== IToken_1.TokenCategory.Comment);
        this.context.emit(line);
    }
}
exports.Filter = Filter;
//# sourceMappingURL=Filter.js.map