"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LabelEmitter = void 0;
const IMessage_1 = require("./interface/IMessage");
const IToken_1 = require("./interface/IToken");
const IParseOptions_1 = require("./interface/IParseOptions");
const PassBase_1 = require("./PassBase");
class LabelEmitter extends PassBase_1.PassBase {
    process(context, options) {
        this.line = 0;
        return super.process(context, options);
    }
    labelName(token) {
        switch (this.options.standard) {
            case IParseOptions_1.Standard.ICWS86:
            case IParseOptions_1.Standard.ICWS88:
                return token.lexeme.length > 8 ? token.lexeme.substr(0, 8) : token.lexeme;
            default:
                return token.lexeme;
        }
    }
    processLine() {
        // Pass 3
        // Replace labels with numbers
        // Raise syntax error for undeclared labels
        if (this.stream.peek().category === IToken_1.TokenCategory.Opcode) {
            this.processLineTokens(true);
            this.line++;
        }
        else if (this.stream.peek().category === IToken_1.TokenCategory.Preprocessor) {
            this.processLineTokens(false);
        }
        else {
            const tokens = this.stream.readToEOL();
            this.context.emit(tokens);
        }
    }
    processLineTokens(isOpcode) {
        const tokens = this.stream.readToEOL();
        tokens.forEach((token) => {
            if (token.category === IToken_1.TokenCategory.Label) {
                this.processLabel(token, isOpcode);
            }
            else {
                this.context.emitSingle(token);
            }
        });
    }
    raiseUndeclaredLabel(label) {
        this.context.messages.push({
            type: IMessage_1.MessageType.Error,
            position: label.position,
            text: "Unrecognised label '" + this.labelName(label) + "'"
        });
    }
    processLabel(label, isOpcode) {
        const name = this.labelName(label);
        if (name in this.context.labels) {
            const labelLine = this.context.labels[name];
            let diff = labelLine;
            if (isOpcode) {
                diff -= this.line;
            }
            const token = {
                category: IToken_1.TokenCategory.Number,
                lexeme: diff.toString(),
                position: Object.assign({}, label.position)
            };
            this.context.emitSingle(token);
        }
        else {
            this.raiseUndeclaredLabel(label);
        }
    }
}
exports.LabelEmitter = LabelEmitter;
//# sourceMappingURL=LabelEmitter.js.map