"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoadParser = void 0;
const IMessage_1 = require("./interface/IMessage");
const IParseOptions_1 = require("./interface/IParseOptions");
const Parser_1 = require("./Parser");
class LoadParser {
    constructor(scanner, filter, syntaxCheck, illegalCommandCheck, modPass) {
        this.scanner = scanner;
        this.filter = filter;
        this.syntaxCheck = syntaxCheck;
        this.illegalCommandCheck = illegalCommandCheck;
        this.modPass = modPass;
    }
    noErrors(context) {
        return !context.messages.some((message) => {
            return message.type === IMessage_1.MessageType.Error;
        });
    }
    parse(document, options) {
        options = Object.assign({}, Parser_1.Parser.DefaultOptions, options || {});
        let context = this.scanner.scan(document, options);
        if (this.noErrors(context)) {
            context = this.filter.process(context, options);
        }
        if (this.noErrors(context)) {
            context = this.syntaxCheck.process(context, options);
        }
        if (options.standard < IParseOptions_1.Standard.ICWS94draft) {
            if (this.noErrors(context)) {
                context = this.illegalCommandCheck.process(context, options);
            }
        }
        if (this.noErrors(context)) {
            context = this.modPass.process(context, options);
        }
        return {
            metaData: context.metaData,
            tokens: context.tokens,
            messages: context.messages,
            success: !context.messages.some(message => message.type === IMessage_1.MessageType.Error)
        };
    }
}
exports.LoadParser = LoadParser;
//# sourceMappingURL=LoadParser.js.map