"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetaDataEmitter = void 0;
const IParseOptions_1 = require("./interface/IParseOptions");
const IToken_1 = require("./interface/IToken");
class MetaDataEmitter {
    process(context, options) {
        this.tokens = [];
        this.emitStandard(options.standard);
        if (context.metaData.name) {
            this.emitName(context.metaData.name);
        }
        if (context.metaData.author) {
            this.emitAuthor(context.metaData.author);
        }
        if (context.metaData.strategy) {
            this.emitStrategy(context.metaData.strategy);
        }
        context.tokens = this.tokens.concat(context.tokens);
        return context;
    }
    emitComment(comment) {
        const commentTokens = [
            {
                category: IToken_1.TokenCategory.Comment,
                lexeme: comment,
                position: { char: 0, line: 0 }
            },
            {
                category: IToken_1.TokenCategory.EOL,
                lexeme: '\n',
                position: { char: 0, line: 0 }
            }
        ];
        this.tokens = this.tokens.concat(commentTokens);
    }
    emitStandard(standard) {
        switch (standard) {
            case IParseOptions_1.Standard.ICWS86:
                this.emitComment(';redcode-86');
                break;
            case IParseOptions_1.Standard.ICWS88:
                this.emitComment(';redcode-88');
                break;
            case IParseOptions_1.Standard.ICWS94draft:
                this.emitComment(';redcode-94');
                break;
        }
    }
    emitName(name) {
        this.emitComment(';name ' + name);
    }
    emitAuthor(author) {
        this.emitComment(';author ' + author);
    }
    emitStrategy(strategy) {
        strategy.split('\n').forEach(s => {
            if (s.trim() === '') {
                return;
            }
            this.emitComment(';strategy ' + s);
        });
    }
}
exports.MetaDataEmitter = MetaDataEmitter;
//# sourceMappingURL=MetaDataEmitter.js.map