"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PreprocessAnalyser = void 0;
const IToken_1 = require("./interface/IToken");
const IMessage_1 = require("./interface/IMessage");
class PreprocessAnalyser {
    process(context, _) {
        // Detect dependencies between EQU expressions
        // Raise circular reference errors
        // Replace references to EQU labels in other EQU label definitions
        this.context = context;
        this.references = {};
        this.collectReferences();
        if (this.noCircularReferences()) {
            this.replaceAllReferences();
        }
        return this.context;
    }
    collectReferences() {
        const keys = Object.keys(this.context.equs);
        keys.forEach((key) => {
            const expression = this.context.equs[key];
            const references = expression.filter((token) => {
                return token.category === IToken_1.TokenCategory.Label && keys.includes(token.lexeme);
            });
            this.references[key] = references.map((token) => {
                return token.lexeme;
            });
        });
    }
    raiseCircularReference(key) {
        this.context.messages.push({
            text: "Circular reference detected in '" + key + "' EQU statement",
            type: IMessage_1.MessageType.Error,
            // TODO proper position
            position: { line: 1, char: 1 }
        });
    }
    noCircularReferences() {
        const keys = Object.keys(this.context.equs);
        let result = true;
        keys.forEach((key) => {
            try {
                const seen = [];
                this.detectCircularReferencesRecursive(key, seen);
            }
            catch (reference) {
                this.raiseCircularReference(key);
                result = false;
            }
        });
        return result;
    }
    detectCircularReferencesRecursive(token, seen) {
        if (seen.includes(token)) {
            throw token;
        }
        seen.push(token);
        this.references[token].forEach((reference) => {
            this.detectCircularReferencesRecursive(reference, seen);
        });
        const i = seen.indexOf(token);
        seen.splice(i, 1);
    }
    replaceAllReferences() {
        const keys = Object.keys(this.context.equs);
        keys.forEach((key) => {
            this.replaceReferences(key);
        });
    }
    replaceReferences(key) {
        const expression = this.context.equs[key];
        const keys = Object.keys(this.context.equs);
        while (expression.some((token) => {
            return token.category === IToken_1.TokenCategory.Label && keys.includes(token.lexeme);
        })) {
            for (let i = 0; i < expression.length; i++) {
                if (expression[i].category === IToken_1.TokenCategory.Label) {
                    const label = expression[i].lexeme;
                    if (keys.includes(label)) {
                        expression.splice(i, 1, ...this.context.equs[label]);
                    }
                }
            }
        }
    }
}
exports.PreprocessAnalyser = PreprocessAnalyser;
//# sourceMappingURL=PreprocessAnalyser.js.map