"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PreprocessEmitter = void 0;
const IToken_1 = require("./interface/IToken");
const PassBase_1 = require("./PassBase");
class PreprocessEmitter extends PassBase_1.PassBase {
    /// <summary>
    /// Perform preprocessor substitutions.
    /// Replace EQU defined labels with corresponding expression
    /// </summary>
    processLine() {
        // Perform preprocessor substitution
        // Insert EQU expressions
        const next = this.stream.peek();
        if (next.category === IToken_1.TokenCategory.Label && next.lexeme in this.context.equs) {
            this.replaceLabel();
        }
        else {
            this.context.emit([this.stream.read()]);
        }
    }
    replaceLabel() {
        const label = this.stream.read();
        const originalExpression = this.context.equs[label.lexeme];
        const expression = originalExpression.map((token) => {
            const clone = Object.assign({}, token);
            clone.position = label.position;
            return clone;
        });
        this.context.emit(expression);
    }
}
exports.PreprocessEmitter = PreprocessEmitter;
//# sourceMappingURL=PreprocessEmitter.js.map