"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Core = void 0;
const ICore_1 = require("./interface/ICore");
const IInstruction_1 = require("./interface/IInstruction");
const IMessage_1 = require("./interface/IMessage");
const clone = require("clone");
class Core {
    constructor(publisher) {
        this.locations = null;
        this.publisher = publisher;
    }
    initialise(options) {
        this.options = options;
        this.cs = this.options.coresize;
        this.allocateMemory();
    }
    getSize() {
        return this.cs;
    }
    wrap(address) {
        address = address % this.cs;
        address = address >= 0 ? address : address + this.cs;
        return address;
    }
    triggerEvent(task, address, accessType) {
        const accessEventArgs = {
            warriorId: task ? task.instance.warrior.internalId : null,
            accessType: accessType,
            address: address
        };
        if (task && task.instance.warrior.data) {
            accessEventArgs['warriorData'] = task.instance.warrior.data;
        }
        this.locations[address].access = accessEventArgs;
        this.publisher.queue({
            type: IMessage_1.MessageType.CoreAccess,
            payload: accessEventArgs
        });
    }
    executeAt(task, address) {
        address = this.wrap(address);
        const instruction = this.locations[address].instruction;
        if (instruction.opcode !== IInstruction_1.OpcodeType.DAT) {
            this.triggerEvent(task, address, ICore_1.CoreAccessType.execute);
        }
        return instruction;
    }
    readAt(task, address) {
        address = this.wrap(address);
        this.triggerEvent(task, address, ICore_1.CoreAccessType.read);
        return this.locations[address].instruction;
    }
    getAt(address) {
        address = this.wrap(address);
        return this.locations[address].instruction;
    }
    getWithInfoAt(address) {
        address = this.wrap(address);
        return this.locations[address];
    }
    setAt(task, address, instruction) {
        address = this.wrap(address);
        this.triggerEvent(task, address, ICore_1.CoreAccessType.write);
        this.locations[address].instruction = instruction;
    }
    allocateMemory() {
        this.locations = [];
        for (let i = 0; i < this.cs; i++) {
            this.locations.push({
                instruction: this.buildDefaultInstruction(i),
                access: {
                    accessType: ICore_1.CoreAccessType.write,
                    address: i
                }
            });
        }
    }
    buildDefaultInstruction(index) {
        const instruction = clone(this.options.initialInstruction);
        instruction.address = index;
        return instruction;
    }
}
exports.Core = Core;
//# sourceMappingURL=Core.js.map