"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EndCondition = void 0;
const IMessage_1 = require("./interface/IMessage");
class EndCondition {
    constructor(publisher) {
        this.publisher = publisher;
    }
    buildRoundResult(outcome, winner = null) {
        const result = {
            winnerId: winner && winner.warrior.internalId,
            outcome
        };
        if (winner && winner.warrior.data) {
            result['winnerData'] = winner.warrior.data;
        }
        return result;
    }
    publishRoundEnd(payload) {
        this.publisher.queue({
            type: IMessage_1.MessageType.RoundEnd,
            payload
        });
    }
    publishProgress(cycle, maximumCycles) {
        this.publisher.queue({
            type: IMessage_1.MessageType.RunProgress,
            payload: {
                runProgress: (cycle / maximumCycles) * 100.0,
                cycle,
                maximumCycles
            }
        });
    }
    check(state) {
        if (state.cycle >= state.options.maximumCycles) {
            const result = this.buildRoundResult('DRAW');
            this.publishRoundEnd(result);
            return result;
        }
        this.publishProgress(state.cycle, state.options.maximumCycles);
        const liveWarriors = state.instances.filter((warrior) => warrior.tasks.length > 0);
        if (state.instances.length === 1) {
            if (liveWarriors.length === 0) {
                const result = this.buildRoundResult('NONE');
                this.publishRoundEnd(result);
                return result;
            }
        }
        else if (liveWarriors.length === 1) {
            const result = this.buildRoundResult('WIN', liveWarriors[0]);
            this.publishRoundEnd(result);
            return result;
        }
        return null;
    }
}
exports.EndCondition = EndCondition;
//# sourceMappingURL=EndCondition.js.map