"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Fetcher = void 0;
class Fetcher {
    getNextFetchContext(state) {
        let wi = state.warriorIndex;
        let instance = state.instances[wi];
        while (this.isDead(instance)) {
            wi = (wi + 1) % state.instances.length;
            instance = state.instances[wi];
        }
        const ti = instance.taskIndex;
        const task = instance.tasks[ti];
        return {
            warriorIndex: wi,
            taskIndex: ti,
            instance: instance,
            task: task
        };
    }
    getNextExecution(state) {
        const context = this.getNextFetchContext(state);
        return {
            warriorId: context.instance.warrior.internalId,
            address: context.task.instructionPointer
        };
    }
    fetch(state, core) {
        const c = this.getNextFetchContext(state);
        state.warriorIndex = (c.warriorIndex + 1) % state.instances.length;
        c.instance.taskIndex = (c.taskIndex + 1) % c.instance.tasks.length;
        const ip = c.task.instructionPointer;
        const instruction = core.executeAt(c.task, ip);
        c.task.instructionPointer = (ip + 1) % state.options.coresize;
        return {
            core: core,
            instructionPointer: ip,
            instruction: instruction,
            taskIndex: c.taskIndex,
            task: c.task,
            warriorIndex: c.warriorIndex,
            instance: c.instance,
            operands: []
        };
    }
    isDead(warrior) {
        return warrior.tasks.length === 0;
    }
}
exports.Fetcher = Fetcher;
//# sourceMappingURL=Fetcher.js.map