"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Loader = void 0;
class Loader {
    constructor(random, core, warriorLoader) {
        this.random = random;
        this.warriorLoader = warriorLoader;
        this.core = core;
    }
    load(warriors, options) {
        const result = [];
        let id = 0;
        warriors.forEach((warrior) => {
            const address = this.getValidAddress(result, options);
            result.push(this.warriorLoader.load(address, warrior, id++));
        });
        return result;
    }
    getValidAddress(instances, options) {
        /* eslint-disable-next-line */
        while (true) {
            const address = this.random.get(options.coresize);
            if (this.isValidAddress(address, instances, options)) {
                return address;
            }
        }
    }
    isValidAddress(address, instances, options) {
        let valid = true;
        const core = this.core;
        const instructionLimitLess1 = options.instructionLimit - 1;
        const minSeparationLess1 = options.minSeparation - 1;
        instances.forEach((w) => {
            let s0 = address;
            let f0 = address + instructionLimitLess1;
            let s1 = w.startAddress - minSeparationLess1;
            let f1 = w.startAddress + minSeparationLess1 + instructionLimitLess1;
            s0 = core.wrap(s0);
            f0 = core.wrap(f0);
            s1 = core.wrap(s1);
            f1 = core.wrap(f1);
            if (s0 <= f0) {
                if (s1 <= f1) {
                    if (s0 <= f1 && s1 <= f0) {
                        valid = false;
                        return;
                    }
                }
                else if (s0 <= f1 || s1 <= f0) {
                    valid = false;
                    return;
                }
            }
            else if (s0 <= f1 || s1 <= f0) {
                valid = false;
                return;
            }
        });
        return valid;
    }
}
exports.Loader = Loader;
//# sourceMappingURL=Loader.js.map