"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OptionValidator = void 0;
const IInstruction_1 = require("./interface/IInstruction");
const IOperand_1 = require("./interface/IOperand");
class OptionValidator {
    validate(state, warriorCount) {
        if (state.coresize < 0) {
            throw Error(OptionValidator.CoresizeNegativeMessage);
        }
        if (state.minSeparation < 0) {
            throw Error(OptionValidator.MinSeparationNegativeMessage);
        }
        if (state.instructionLimit < 0) {
            throw Error(OptionValidator.InstructionLimitNegativeMessage);
        }
        const requiredSize = warriorCount * (state.minSeparation + state.instructionLimit);
        if (state.coresize < requiredSize) {
            throw Error(OptionValidator.CoreTooSmallForWarriorsMessage);
        }
        if (state.maximumCycles < 1) {
            throw Error(OptionValidator.MaximumCyclesNegativeMessage);
        }
        if (state.maxTasks < 1) {
            throw Error(OptionValidator.MaxTasksNegativeMessage);
        }
        if (!state.initialInstruction) {
            throw Error(OptionValidator.NoInitialInstructionMessage);
        }
        if (!Object.values(IInstruction_1.OpcodeType).includes(state.initialInstruction.opcode)) {
            throw Error(OptionValidator.InitialInstructionOpcodeMessage);
        }
        if (!Object.values(IInstruction_1.ModifierType).includes(state.initialInstruction.modifier)) {
            throw Error(OptionValidator.InitialInstructionModifierMessage);
        }
        if (!state.initialInstruction.aOperand) {
            throw Error(OptionValidator.InitialInstructionAOperandMessage);
        }
        if (!state.initialInstruction.bOperand) {
            throw Error(OptionValidator.InitialInstructionBOperandMessage);
        }
        if (!Object.values(IOperand_1.ModeType).includes(state.initialInstruction.aOperand.mode)) {
            throw Error(OptionValidator.InitialInstructionAOperandModeMessage);
        }
        if (!Object.values(IOperand_1.ModeType).includes(state.initialInstruction.bOperand.mode)) {
            throw Error(OptionValidator.InitialInstructionBOperandModeMessage);
        }
    }
}
exports.OptionValidator = OptionValidator;
OptionValidator.CoresizeNegativeMessage = 'Coresize must be a positive integer';
OptionValidator.MinSeparationNegativeMessage = 'MinSeparation must be a positive integer';
OptionValidator.InstructionLimitNegativeMessage = 'InstructionLimit must be a positive integer';
OptionValidator.CoreTooSmallForWarriorsMessage = 'The coresize is too small for all warriors to be placed. Coresize must be greater than minSeparation plus instructionLimit multiplied by number of warriors';
OptionValidator.MaximumCyclesNegativeMessage = 'maximumCycles must be an integer greater than zero';
OptionValidator.MaxTasksNegativeMessage = 'maxTasks must be an integer greater than zero';
OptionValidator.NoInitialInstructionMessage = 'No initial instruction specified';
OptionValidator.InitialInstructionOpcodeMessage = 'Initial instruction must have a valid opcode';
OptionValidator.InitialInstructionModifierMessage = 'Initial instruction must have a valid modifier';
OptionValidator.InitialInstructionAOperandModeMessage = 'Initial instruction must have a valid a operand address mode';
OptionValidator.InitialInstructionAOperandMessage = 'Initial instruction must have a valid a operand';
OptionValidator.InitialInstructionBOperandModeMessage = 'Initial instruction must have a valid b operand address mode';
OptionValidator.InitialInstructionBOperandMessage = 'Initial instruction must have a valid b operand';
//# sourceMappingURL=OptionValidator.js.map