"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Publisher = void 0;
const clone = require("clone");
class Publisher {
    constructor(strategies) {
        this.publishStrategies = Object.keys(strategies).reduce((a, c) => (Object.assign(Object.assign({}, a), { [c]: clone(strategies[c]) })), {});
        this.republishStrategies = Object.keys(strategies).reduce((a, c) => (Object.assign(Object.assign({}, a), { [c]: clone(strategies[c]) })), {});
    }
    setPublishProvider(publishProvider) {
        this.publishProvider = publishProvider;
    }
    queue(message) {
        this.publishStrategies[message.type].queue(message);
        this.republishStrategies[message.type].queue(message);
    }
    publish() {
        this.doPublish(this.publishStrategies);
        Object.keys(this.publishStrategies).forEach(key => {
            this.publishStrategies[key].clear();
        });
    }
    republish() {
        this.doPublish(this.republishStrategies);
    }
    doPublish(strategies) {
        if (!this.publishProvider) {
            return;
        }
        Object.keys(strategies).forEach(key => {
            const strategy = strategies[key];
            const message = strategy.dequeue();
            if (!message) {
                return;
            }
            this.publishProvider.publishSync(message.type, message.payload);
        });
    }
    clear() {
        Object.keys(this.publishStrategies).forEach(key => {
            this.publishStrategies[key].clear();
        });
        Object.keys(this.republishStrategies).forEach(key => {
            this.republishStrategies[key].clear();
        });
    }
}
exports.Publisher = Publisher;
//# sourceMappingURL=Publisher.js.map